import version as v
import os
import sys
import subprocess
from string import Template

rc_template = """2 ICON "src/icons/horizon-eda.ico"

1 VERSIONINFO
FILEVERSION     $major,$minor,$micro,0
PRODUCTVERSION  $major,$minor,$micro,0
BEGIN
  BLOCK "StringFileInfo"
  BEGIN
    BLOCK "040904E4"
    BEGIN
      VALUE "FileDescription", "Horizon EDA"
      VALUE "InternalName", "horizon-eda"
      VALUE "OriginalFilename", "horizon-eda.exe"
      VALUE "ProductName", "Horizon EDA"
      VALUE "ProductVersion", "$major.$minor.$micro"
    END
  END
  BLOCK "VarFileInfo"
  BEGIN
    VALUE "Translation", 0x809, 1252
  END
END
"""

outfile = sys.argv[1]
with open(outfile, "w", encoding="utf-8") as fi:
	tmpl = Template(rc_template)
	fi.write(tmpl.substitute(major=v.major, minor=v.minor, micro=v.micro))



import sys
import yaml
sys.path.append("build")
import horizon

types = (
	"unit",
	"symbol",
	"entity",
	"part",
	"padstack",
	"package",
	"frame",
	"decal",
	"project",
	"board",
	"schematic",
	"blocks",
)

version_yml = yaml.load(open("scripts/app_versions.yml", "r"), Loader=yaml.SafeLoader)["versions"]

exitcode = 0

for object_type in types:
	app_version = horizon.get_app_version(object_type)
	if app_version != 0 :
		if object_type not in version_yml :
			print(object_type, "not found")
			exitcode = 1
			continue
		for v in range(1, app_version+1) :
			if v not in version_yml[object_type] :
				print(object_type, "version", v, "not found")
				exitcode = 1

if exitcode == 0 :
	print("All app versions okay")

exit(exitcode)



import xml.etree.ElementTree as ET
import sys
import json
#use this script with the XML files from the db/mcu directory in an STM32CubeMX installation

ns = {'st': 'http://mcd.rou.st.com/modules.php?name=mcu'}
type_dir_map = {
	"Power": "power_input",
	"Reset": "input"
}

tree = ET.parse(sys.argv[1])
root = tree.getroot()
out = {}

for pin in root.findall("st:Pin", ns) :
	pin_name = pin.get("Name")
	pin_pos = pin.get("Position")
	pin_type = pin.get("Type")
	pin_dir = type_dir_map.get(pin_type, "bidirectional")

	alt_names = []
	for alt in pin.findall("st:Signal", ns) :
		n = alt.get("Name")
		if n != "GPIO":
			alt_names.append(n)

	out[pin_pos] = {"pin": pin_name, "direction": pin_dir, "alt":alt_names}

print (json.dumps(out, sort_keys=True, indent=4, separators=(',', ': ')))



import sys
mode = sys.argv[1]
infile = sys.argv[2]

texts = {}
text_current = None
for line in open(infile, "r", encoding="UTF-8") :
	line = line.strip()
	if line.startswith("#") :
		text_current = line[1:]
		texts[text_current] = ""
	elif text_current is not None :
		texts[text_current] += line+"\n"

texts = {k:v.strip() for k,v in texts.items()}

if mode == 'h' :
	print("#pragma once")
	print("namespace horizon {")
	print("class HelpTexts {")
	print("public:")
	for k in texts.keys() :
		print("    static const char* %s;"%k)
	print("};")
	print("}")

if mode == 'c' :
	print('#include "help_texts.hpp"')
	print("namespace horizon {")
	
	for k,v in texts.items() :
		print('const char* HelpTexts::%s = R"EOF(%s)EOF";'%(k,v))
	print("}")



#!/usr/bin/python3
import re
import pygit2

repo = pygit2.Repository(".")

def rep(match) :
	m = match.group(0)
	if m.startswith("@"):
		return f"[{m}](https://github.com/{m[1:]})"
	else :
		commit,_ = repo.resolve_refish(m)
		return "[%s](https://github.com/horizon-eda/horizon/commit/%s)"%(commit.short_id, m)

with open("scripts/CHANGELOG.md.in", "r") as infile :
	changelog = infile.read()
	with open("CHANGELOG.md", "w") as outfile :
		outfile.write(re.sub(r"\b[0-9a-f]{40}|@\w+\b", rep, changelog))



#!/usr/bin/python
import sys
sys.path.append("..")
import version as v

import os
import struct


version = v.string
versionstr = v.string
architecture = '-x64'



# required by install.bat
print(versionstr + architecture)
	

with open('version.wxi', 'w') as wxi:
	wxi.write("<?xml version='1.0' encoding='utf-8'?>\n")
	wxi.write("<!-- do not edit, this file is created by version.py tool any changes will be lost -->\n")
	wxi.write("<Include>\n")
	wxi.write("<?define ProductVersion='" + version + "' ?>\n")
	wxi.write("<?define FullProductName='Horizon EDA' ?>\n")

	wxi.write("<?define ProgramFilesFolder='ProgramFiles64Folder' ?>\n")
	wxi.write("<?define Win64='yes' ?>\n")
	wxi.write("<?define InstallerVersion='200' ?>\n")
	wxi.write("<?define Platform='x64' ?>\n")

	wxi.write("</Include>\n")




import version as v
import xml.etree.ElementTree as ET
tree = ET.parse('org.horizon_eda.HorizonEDA.metainfo.xml')
root = tree.getroot()
version_from_xml = root.find("releases").find("release").attrib["version"]

rc = 0
if version_from_xml == v.string :
	print("Version okay")
else:
	print("Version mismatch %s != %s"%(v.string, version_from_xml))
	rc = 1

#Check changelog versions

for filename in ("CHANGELOG.md", "scripts/CHANGELOG.md.in") :
	first_line = next(open(filename, "r")).strip()
	if first_line != f"# Version {v.string}" :
		print(f"{filename} version mismatch")
		rc = 1
	else :
		print(f"{filename} version okay")

exit(rc)



import version as v
import os
import sys
import subprocess
from string import Template

cpp_template = """#include "util/version.hpp"
namespace horizon {
const unsigned int Version::major = $major;
const unsigned int Version::minor = $minor;
const unsigned int Version::micro = $micro;
const char *Version::name = "$name";
const char *Version::commit = "$commit";
const char *Version::commit_hash = "$commit_hash";
}
"""

gitversion = ""
commit_hash = ""
if os.path.isdir(".git"):
	gitversion = subprocess.check_output(['git', 'log', '-1', '--pretty=format:%h %ci %s']).decode()
	gitversion = gitversion.replace("\"", "\\\"")
	commit_hash = subprocess.check_output(['git', 'log', '-1', '--pretty=format:%h']).decode()

outfile = sys.argv[1]
with open(outfile, "w", encoding="utf-8") as fi:
	tmpl = Template(cpp_template)
	fi.write(tmpl.substitute(major=v.major, minor=v.minor, micro=v.micro, string=v.string, name=v.name, commit=gitversion, commit_hash=commit_hash))



#!/usr/bin/python

from __future__ import print_function
from __future__ import unicode_literals # make all literals unicode strings by default (even in Python 2)

import os
import re
import uuid
from io import open # needed for support of encoding parameter in Python 2


# basestring is not available in Python 3
try:
  basestring
except NameError:
  basestring = (str,bytes)


directory_ids = {}
file_ids = {}

def indent(level):
	indentstring = ''
	for i in range(level):
		indentstring += '	'
	return indentstring

def valid_id(identifier):
	return identifier.replace('@','_')

def directory(root, breadcrumb, level, exclude=[]):
	"""
	list all files and directory recursively
	create the file_ids dictionary to be used in ComponentGroup references
	"""
	global file_ids
	global directory_ids
	# first list files within directory
	files = [ f for f in os.listdir(root) if os.path.isfile(os.path.join(root,f)) and f not in exclude]
	for file in files:
		file_key = os.path.join(root, file)
		file_key = file_key.replace('/', '\\')  # for usage from MSYS2 shell
		_id = '_%06d' % (len(file_ids.keys()) + 1)
		file_ids[file_key] = 'component' + _id
		wxs.write(indent(level)+ "<Component Id='component" + _id + "' Guid='" + str(uuid.uuid4()) + "' DiskId='1' Win64='$(var.Win64)'>\n")
		if file == 'horizon-eda.exe':
			_id = '_horizon_exe'
		wxs.write(indent(level + 1)+ "<File Id='file" + _id + "' Name='" + file + "' DiskId='1' Source='" + file_key + "' KeyPath='yes' />\n")
		wxs.write(indent(level)+ "</Component>\n")

	# then all directories
	dirs = [ f for f in os.listdir(root) if os.path.isdir(os.path.join(root,f)) ]
	for dir in dirs:
		directory_key = breadcrumb + '__' + dir
		if not directory_key in directory_ids.keys():
			directory_ids[directory_key] = 'dir_%06d' % (len(directory_ids.keys()) + 1)
		wxs.write(indent(level) + "<Directory Id='" + directory_ids[directory_key] + "' Name='" + dir + "'>\n")
		directory(os.path.join(root, dir), directory_key, level + 1)
		wxs.write(indent(level) + "</Directory>\n")

def test_conditions(value, conditions):
	"""
	check if "value" fulfills any of the "conditions", where a condition can be
	   - a string that has to be a substring of "value"
	   - a compiled regex pattern which has to match in "value"
	"""
	if not isinstance(conditions, list):
		conditions = [conditions]
	for condition in conditions:
		if isinstance(condition, basestring):
			if condition in value:
				return True
		elif isinstance(condition, type(re.compile(''))):
			if re.search(condition, value):
				return True
	return False


def ComponentGroup(name, conditions, level):
	"""
	add componentgroup that contain all items from file_ids that match condition
	remove the matched elements from file_ids
	"""
	global file_ids
	keys = [k for k in file_ids.keys() if test_conditions(k, conditions)]
	wxs.write(indent(level) + "<ComponentGroup Id='" + name + "'>\n")
	for component in keys:
		wxs.write(indent(level + 1) + "<ComponentRef Id='" + file_ids[component] + "' />\n")
	wxs.write(indent(level) + "</ComponentGroup>\n")
	for key in keys:
		del file_ids[key]

# get directory containing the distribution files
horizon_dist_dir = "../dist/horizon"


with open('files.wxs', 'w', encoding='utf-8') as wxs:
	wxs.write("<!-- do not edit, this file is created by files.py tool any changes will be lost -->\n")
	wxs.write("<Wix xmlns='http://schemas.microsoft.com/wix/2006/wi'>\n")
	wxs.write(indent(1) + "<?include version.wxi?>\n")
	wxs.write(indent(1) + "<Fragment>\n")
	wxs.write(indent(2) + "<!-- Step 1: Define the directory structure -->\n")
	wxs.write(indent(2) + "<Directory Id='TARGETDIR' Name='SourceDir'>\n")
	wxs.write(indent(3) + "<Directory Id='$(var.ProgramFilesFolder)' Name='PFiles'>\n")
	wxs.write(indent(4) + "<Directory Id='INSTALLDIR' Name='Horizon EDA'>\n")
	print("start parsing files from " + horizon_dist_dir)
	directory(horizon_dist_dir, 'horizon', 5)
	print("found %d files" % len(file_ids.keys()))
	wxs.write(indent(4) + "</Directory>\n")
	wxs.write(indent(3) + "</Directory>\n")
	# link to ProgramMenu
	wxs.write(indent(3) + "<Directory Id='ProgramMenuFolder'>\n")
	wxs.write(indent(4) + "<Directory Id='ApplicationProgramsFolder' Name='$(var.FullProductName)'/>\n")
	wxs.write(indent(3) + "</Directory>\n")
	wxs.write(indent(3) + "<Directory Id='DesktopFolder' Name='Desktop' />\n")
	wxs.write(indent(2) + "</Directory>\n")

	ComponentGroup("AllOther", '', 2)

	wxs.write(indent(1) + "</Fragment>\n")
	wxs.write("</Wix>\n")
